using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;
using CoreLab.MySql;
using System.Collections;
using System.Data;

namespace CustomSettingsProvider.Provider
{
    public class MySqlSettingsProvider : SettingsProvider
    {
        #region Fields

        string _applicationName = "MySqlSettingsProvider";
        MySqlConnection _connection = new MySqlConnection();
        string _clientID = "";
        String _settingTable = ""; 

        #endregion

        #region Constructors
        public MySqlSettingsProvider()
        {
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets the name of the currently running application.
        /// </summary>
        /// <value></value>
        /// <returns>Ein <see cref="T:System.String"></see> mit dem verkrzten Namen der Anwendung, der keinen vollstndigen Pfad oder keine Erweiterung enthlt, z.B. SimpleAppSettings.</returns>
        public override string ApplicationName
        {
            get { return this._applicationName; }
            set { this._applicationName = value; }
        }

        /// <summary>
        /// Gets the friendly name used to refer to the provider during configuration
        /// </summary>
        /// <value></value>
        /// <returns>Friendly name used to refer to the provider during configuration</returns>
        public override string Name
        {
            get { return "MySqlSettingsProvider"; }
        }
        #endregion

        #region Methods

        /// <summary>
        /// Initializes the provider
        /// </summary>
        public override void Initialize(string name, System.Collections.Specialized.NameValueCollection col)
        {
            base.Initialize(this.ApplicationName, col);
        }


        /// <summary>
        /// Returns the collection of settings property values for the specified application instance and settings property group. 
        /// </summary>
        /// <param name="context">The context.</param>
        /// <param name="propertyCollection">The property collection.</param>
        /// <returns></returns>
        public override SettingsPropertyValueCollection GetPropertyValues(SettingsContext context, SettingsPropertyCollection propertyCollection)
        {
            this.PrepareContextSettings(context);


            #region Retrieve Settings from DB

            string sSelectCmd = String.Format("SELECT * FROM {0} WHERE ClientID = '' OR ClientID = '{1}' ORDER BY ClientID DESC", _settingTable, _clientID);

            MySqlDataAdapter daSettings = new MySqlDataAdapter(sSelectCmd, _connection);
            DataTable dtSettings = new DataTable();
            daSettings.Fill(dtSettings);
            

            #endregion


            // Create new collection of values
            SettingsPropertyValueCollection spValueCollection = new SettingsPropertyValueCollection();

            #region Iterate through the settings to be retrieved

            foreach (SettingsProperty property in propertyCollection)
            {
                SettingsPropertyValue value = new SettingsPropertyValue(property);

                string sFilter = "PropertyName = '" + property.Name + "'";

                if (property.Attributes[typeof(System.Configuration.UserScopedSettingAttribute)] != null)
                    sFilter += " AND ClientID = '" + _clientID + "'";
                else
                    sFilter += " AND ClientID = ''";

                DataRow[] arDr = dtSettings.Select(sFilter);
                if (arDr.Length > 0) // Setting in DB available
                {
                    value.SerializedValue = arDr[0]["PropertyValue"];
                    value.IsDirty = false;
                }
                else // not in DB yet
                {
                    value.SerializedValue = value.PropertyValue;
                    value.IsDirty = true;
                }

                spValueCollection.Add(value);
            }

            #endregion

            return spValueCollection;
        }

        /// <summary>
        /// Sets the values of the specified group of property settings. 
        /// </summary>
        /// <param name="context">The context.</param>
        /// <param name="propertyValueCollection">The property value collection.</param>
        public override void SetPropertyValues(SettingsContext context, SettingsPropertyValueCollection propertyValueCollection)
        {
            this.PrepareContextSettings(context);

            #region Prepare MySQL Command

            MySqlCommand cmd = new MySqlCommand("REPLACE INTO " + _settingTable + " (PropertyName, ClientID, PropertyValue, Version) VALUES (:PropertyName, :ClientID, :PropertyValue, :Version)", _connection);
            cmd.Parameters.Add("PropertyName", "");
            cmd.Parameters.Add("ClientID", "");
            cmd.Parameters.Add("PropertyValue", "");
            cmd.Parameters.Add("Version", "");

            #endregion

            // Iterate through the settings to be stored
            // Only IsDirty==true properties should be stored

            _connection.Open();
            string sValue;
            foreach (SettingsPropertyValue propertyValue in propertyValueCollection)
            {
                if (propertyValue.IsDirty)
                {
                    try
                    {
                        if (propertyValue.SerializedValue == null)
                            continue;

                        sValue = propertyValue.SerializedValue.ToString();

                        cmd.Parameters["PropertyName"].Value = propertyValue.Name;
                        cmd.Parameters["PropertyValue"].Value = sValue;

                        //Context: ClientID
                        if (propertyValue.Property.Attributes[typeof(System.Configuration.UserScopedSettingAttribute)] != null)
                            cmd.Parameters["ClientID"].Value = context["ClientID"].ToString();
                        else
                            cmd.Parameters["ClientID"].Value = "";

                        //Custom Attribute: VersionAttribute 
                        //if (htAttributes.Contains("CustomSettingsProvider.VersionAttribute"))
                        if (propertyValue.Property.Attributes[typeof(CustomSettingsProvider.VersionAttribute)] != null)
                             cmd.Parameters["Version"].Value = ((VersionAttribute)propertyValue.Property.Attributes[typeof(CustomSettingsProvider.VersionAttribute)]).Version;
                        else
                            cmd.Parameters["Version"].Value = "";

                        cmd.ExecuteScalar();
                    }
                    catch (Exception ex)
                    {
                        Console.WriteLine(ex);
                    }
                }
            }
            _connection.Close();
        }

        private void PrepareContextSettings(SettingsContext context)
        {
            if (!context.Contains("SettingConnectionString") || !context.Contains("SettingTable") || !context.Contains("ClientID"))
            {
                throw new ConfigurationErrorsException("The context of the settings-object must contain these keys: SettingConnectionString, SettingTable, ClientID");
            }

            this._connection.ConnectionString = context["SettingConnectionString"].ToString();
            this._clientID = context["ClientID"].ToString();
            this._settingTable = context["SettingTable"].ToString();
        }
        #endregion
    }
}
